// BarcodeReader.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include "../SoftekBarcodeDLL.h"

// Uncomment if using mtScanBarCodeFromString
//#include <fstream>
//#include <vector>

int _tmain(int argc, _TCHAR* argv[])
{
	// Create a handle to the barcode SDK
	void *hBarcode = mtCreateBarcodeInstance();

	// Get a 30-day key from sales@bardecode.com and enter below to suppress the pop up box
	// mtSetLicenseKey(hBarcode, "MY LICENSE KEY");

	// Turn on the barcode types you want to read.
	// Turn off the barcode types you don't want to read (this will increase the speed of your application)
	mtSetReadCode128(hBarcode, true);
	mtSetReadCode39(hBarcode, true);
	mtSetReadCode25(hBarcode, true);
	mtSetReadEAN13(hBarcode, true);
	mtSetReadEAN8(hBarcode, true);
	mtSetReadUPCA(hBarcode, true);
	mtSetReadUPCE(hBarcode, true);
	mtSetReadCodabar(hBarcode, true);
	mtSetReadPDF417(hBarcode, true);
	mtSetReadDataMatrix(hBarcode, true);
	mtSetReadDatabar(hBarcode, true);
	mtSetReadMicroPDF417(hBarcode, true);
	mtSetReadQRCode(hBarcode, true);

	// Databar Options is a mask that controls which type of databar barcodes will be read and whether or not
	// the software will look for a quiet zone around the barcode.
	// 1 = 2D-Linkage flag (handle micro-PDf417 barcodes as supplementary data - requires ReadMicroPDF417 to be true).
	// 2 = Read RSS14
	// 4 = Read RSS14 Stacked
	// 8 = Read Limited
	// 16 = Read Expanded
	// 32 = Read Expanded Stacked
	// 64 = Require quiet zone
	mtSetDatabarOptions(hBarcode, 255);

	// If you want to read more than one barcode then set Multiple Read to 1
	// Setting MutlipleRead to False will make the recognition faster
	mtSetMultipleRead(hBarcode, true);

	// Noise reduction takes longer but can make it possible to read some difficult barcodes
	// When using noise reduction a typical value is 10 - the smaller the value the more effect it has.
	// A zero value turns off noise reduction. 
	// mtSetNoiseReduction(hBarcode, 0 );

	// You may need to set a small quiet zone if your barcodes are close to text and pictures in the image.
	// A value of zero uses the default.
	mtSetQuietZoneSize(hBarcode, 0);

	// LineJump controls the frequency at which scan lines in an image are sampled.
	// The default is 9 - decrease this for difficult barcodes.
	mtSetLineJump(hBarcode, 1);

	// You can restrict your search to a particular area of the image if you wish.
	// This example limits the search to the upper half of the page
	// System.Drawing.Rectangle scanArea(hBarcode, new System.Drawing.Rectangle(0, 0, 100, 50));
	// mtSetSetScanRect(scanArea, 1));

	// Set the direction that the barcode reader should scan for barcodes
	// The value is a mask where 1(hBarcode, Left to Right, 2(hBarcode, Top to Bottom, 4(hBarcode, Right To Left, 8(hBarcode, Bottom to Top
	mtSetScanDirection(hBarcode, 15);

	// DeskewMode controls the way in which the toolkit handles skewed documents. At the default level (2) 
	// the toolkit will deskew at the main angle of the whole image. If the bar code is at a different angle
	// to the rest of the image then try 3. For more complex images try values from 4 to 6.
	mtSetDeskewMode(hBarcode, 2);

	// ColorProcessingLevel controls how much time the toolkit will searching a color image for a mtSet
	// The default value is 2 and the range of values is 0 to 5. If ColorThreshold is non-zero then 
	// ColorProcessingLevel is effectively set to 0.
	mtSetColorProcessingLevel(hBarcode, 2);

	// MaxLength and MinLength can be used to specify the number of characters you expect to
	// find in a mtSet This can be useful to increase accuracy or if you wish to ignore some
	// barcodes in an image.
	mtSetMinLength(hBarcode, 4);
	mtSetMaxLength(hBarcode, 999);

	// When the toolkit scans an image it records the score it gets for each barcode that
	// MIGHT be in the image. If the scores recorded for any of the barcodes are >= PrefOccurrence
	// then only these barcodes are returned. Otherwise, any barcode whose scores are >= MinOccurrence
	// are reported. If you have a very poor quality image then try setting MinOccurrence to 1, but you
	// may find that some false positive results are returned.
	// mtSetMinOccurrence(hBarcode, 2 );
	// mtSetPrefOccurrence(hBarcode, 4 );

	// Flags for handling PDF files
	// PdfImageOnly defaults to true and indicates that the PDF documents are simple images.
	mtSetPdfImageOnly(hBarcode, true);

	// The PdfImageExtractOptions mask controls how images are removed from PDF documents (when PdfImageOnly is True)
	// 1 = Enable fast extraction
	// 2 = Auto-invert black and white images
	// 4 = Auto-merge strips
	// 8 = Auto-correct photometric values in black and white images
	mtSetPdfImageExtractOptions(hBarcode, 15);

	// The PdfImageRasterOptions mask controls how images are rasterized when PdfImageOnly is false or when image extraction fails
	// 1 = Use alternative pdf-to-tif conversion function
	// 2 = Always use pdf-to-tif conversion rather than loading the rasterized image directly into memory
	mtSetPdfImageRasterOptions(hBarcode, 0);

	// PdfDpi and PdfBpp control what sort of image the PDF document is rasterized into
	mtSetPdfDpi(hBarcode, 300);
	mtSetPdfBpp(hBarcode, 8);

	// Set some properties
	mtSetReadQRCode(hBarcode, true); // Read QR Codes
	mtSetMinLength(hBarcode, 3); // Set a minimum length of 3 (default is 4)

	int n = mtScanBarCode(hBarcode, "../Images/qrcode/qrcode01.jpg");

	/* Or load into a buffer and use mtScanBarCodeFromString */
	/* Uncomment the 2 #include lines at the top of this file */

	/*
	std::ifstream file("../Images/qrcode/qrcode01.jpg", std::ios::binary | std::ios::ate);
	std::streamsize size = file.tellg();

	file.seekg(0, std::ios::beg);

	std::vector<char> buffer(size);
	file.read(buffer.data(), size);

	int n = mtScanBarCodeFromString(hBarcode, (unsigned char *) buffer.data(), buffer.size());
	*/

	if (n < 0) {
		printf ("Error code %d, SDK error %d\n", n, mtGetLastError(hBarcode));
	} else if (n == 0) {
		printf ("No barcodes found in this image\n");
	} else {
		for (int i = 1; i <= n; i++) {
			printf ("Barcode %d\n", i);
			printf ("Value = %s\n", mtGetBarString(hBarcode, i));
			printf ("Type = %s\n", mtGetBarStringType(hBarcode, i));
			printf ("Quality score = %d\n", mtGetBarStringQualityScore(hBarcode, i));
			int nDirection = mtGetBarStringDirection(hBarcode, i);
			if (nDirection == 1)
				printf ("Direction = Left to Right\n");
			else if (nDirection == 2)
				printf ("Direction = Top to Bottom\n");
			else if (nDirection == 4)
				printf ("Direction = Right to Left\n");
			else if (nDirection == 8)
				printf ("Direction = Bottom to Top\n");
			printf("Page = %d\n", mtGetBarStringPage(hBarcode, i));
			long tlx, tly, brx, bry;
			mtGetBarStringPos(hBarcode, i, &tlx, &tly, &brx, &bry);
			printf("Top Left = %ld, %ld\n", tlx, tly);
			printf("Bottomright = %ld, %ld\n", brx, bry);
			printf("\n");
		}
	}

	mtDestroyBarcodeInstance(hBarcode);

	return 0;
}

