﻿
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Drawing;
using System.Diagnostics;
using System.IO;

namespace SoftekBarcodeNet
{
    public class BarcodeReader : IDisposable
    {
		// Assumes 64-bit DLL if not specifically x86
#if x86
	const string softekLibPath = "SoftekBarcodeDLL.dll";
#else
	const string softekLibPath = "SoftekBarcode64DLL.dll";
#endif

		// External dll declarations

		[DllImport(softekLibPath)]
        private static extern short mtGetLineJump(IntPtr hbarcode);
	[DllImport(softekLibPath)]
        private static extern IntPtr mtCreateBarcodeInstance();
	[DllImport(softekLibPath)]
	private static extern IntPtr mtDestroyBarcodeInstance(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtGetBitmapResolution(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetBitmapResolution(IntPtr hBarcode, short newValue);
	[DllImport(softekLibPath)]
	private static extern IntPtr mtGetDebugTraceFile(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern IntPtr mtSetDebugTraceFile(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] lpszNewValue);
	[DllImport(softekLibPath)]
	private static extern IntPtr mtSetLicenseKey(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] lpszNewValue);
	[DllImport(softekLibPath)]
	private static extern IntPtr mtGetLicenseKey(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetLineJump(IntPtr hBarcode, short nNewValue);
	[DllImport(softekLibPath)]
	private static extern double mtGetLowerRatio(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern double mtSetLowerRatio(IntPtr hBarcode, double newValue);
	[DllImport(softekLibPath)]
	private static extern short mtGetMinOccurrence(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetMinOccurrence(IntPtr hBarcode, short nNewValue);
	[DllImport(softekLibPath)]
	private static extern bool mtGetMultipleRead(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetMultipleRead(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern short mtGetNoiseReduction(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetNoiseReduction(IntPtr hBarcode, short nNewValue);
	[DllImport(softekLibPath)]
	private static extern short mtGetPageNo(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetPageNo(IntPtr hBarcode, short nNewValue);
	[DllImport(softekLibPath)]
	private static extern short mtGetPrefOccurrence(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetPrefOccurrence(IntPtr hBarcode, short nNewValue);
	[DllImport(softekLibPath)]
	private static extern short mtGetQuietZoneSize(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetQuietZoneSize(IntPtr hBarcode, short nNewValue);
	[DllImport(softekLibPath)]
	private static extern short mtGetRotation(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetRotation(IntPtr hBarcode, short nNewValue);
	[DllImport(softekLibPath)]
	private static extern double mtGetUpperRatio(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern double mtSetUpperRatio(IntPtr hBarcode, double newValue);
	[DllImport(softekLibPath)]
	private static extern IntPtr mtGetBarString(IntPtr hBarcode, short index);
	[DllImport(softekLibPath)]
	private static extern int mtGetBarStringA(IntPtr hBarcode, short index, [MarshalAs(UnmanagedType.LPArray)] byte[] pBytes, int nBytes);
	[DllImport(softekLibPath)]
	private static extern int mtGetRawBarString(IntPtr hBarcode, short index, [MarshalAs(UnmanagedType.LPArray)] byte[] pBytes, int nBytes);
	[DllImport(softekLibPath)]
	private static extern short mtScanBarCode(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] file);
	[DllImport(softekLibPath)]
	private static extern short mtSetPDFTool(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] pdfTool);
	[DllImport(softekLibPath)]
	private static extern IntPtr mtGetPDFTool(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtScanBarCodeFromDIB(IntPtr hBarcode, IntPtr hDIB);
	[DllImport(softekLibPath)]
	private static extern short mtScanBarCodeFromBitmap(IntPtr hBarcode, IntPtr hBitmap);
	[DllImport(softekLibPath)]
	private static extern short mtScanBarCodeFromBitmapA(IntPtr hBarcode, int bmType, int bmWidth, int bmHeight, int bmWidthBytes, int bmPlanes, int bmBitsPixel, IntPtr bmBits);
	[DllImport(softekLibPath)]
	private static extern bool mtGetReadCode39(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetReadCode39(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern bool mtGetReadEAN13(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetReadEAN13(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern bool mtGetReadEAN8(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetReadEAN8(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern bool mtGetReadUPCA(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetReadUPCA(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern bool mtGetReadUPCE(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetReadUPCE(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern bool mtGetShowCheckDigit(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetShowCheckDigit(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern IntPtr mtGetBarStringType(IntPtr hBarcode, short index);
	[DllImport(softekLibPath)]
	private static extern bool mtGetReadCode128(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetReadCode128(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern bool mtGetReadShortCode128(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetReadShortCode128(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern bool mtGetCode39NeedStartStop(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetCode39NeedStartStop(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern short mtGetColorThreshold(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetColorThreshold(IntPtr hBarcode, short nNewValue);
	[DllImport(softekLibPath)]
	private static extern int mtGetBarStringPage(IntPtr hBarcode, short nBarCode);
	[DllImport(softekLibPath)]
	private static extern int mtGetBarStringTlx(IntPtr hBarcode, short nBarCode);
	[DllImport(softekLibPath)]
	private static extern int mtGetBarStringTly(IntPtr hBarcode, short nBarCode);
	[DllImport(softekLibPath)]
	private static extern int mtGetBarStringBrx(IntPtr hBarcode, short nBarCode);
	[DllImport(softekLibPath)]
	private static extern int mtGetBarStringBry(IntPtr hBarcode, short nBarCode);
	[DllImport(softekLibPath)]
	private static extern bool mtGetReadCode25(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetReadCode25(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern bool mtSetScanRectA(IntPtr hBarcode, Int32 TopLeftX, Int32 TopLeftY, Int32 BottomRightX, Int32 BottomRightY, short MappingMode);
	[DllImport(softekLibPath)]
	private static extern bool mtGetReadPatchCodes(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetReadPatchCodes(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern bool mtGetUseOverSampling(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetUseOverSampling(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern short mtGetScanDirection(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetScanDirection(IntPtr hBarcode, short sNewValue);
	[DllImport(softekLibPath)]
	private static extern short mtGetMinLength(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetMinLength(IntPtr hBarcode, short sNewValue);
	[DllImport(softekLibPath)]
	private static extern short mtGetShortCode128MinLength(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetShortCode128MinLength(IntPtr hBarcode, short sNewValue);
	[DllImport(softekLibPath)]
	private static extern short mtGetMaxLength(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetMaxLength(IntPtr hBarcode, short sNewValue);
	[DllImport(softekLibPath)]
	private static extern IntPtr mtSetTifSplitPath(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] lpszNewValue);
	[DllImport(softekLibPath)]
	private static extern IntPtr mtGetTifSplitPath(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern IntPtr mtSetTifSplitValues(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] lpszNewValue);
	[DllImport(softekLibPath)]
	private static extern IntPtr mtGetTifSplitValues(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtGetTifSplitMode(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetTifSplitMode(IntPtr hBarcode, short sNewValue);
	[DllImport(softekLibPath)]
	private static extern bool mtGetReadCodabar(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetReadCodabar(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern short mtGetBarStringDirection(IntPtr hBarcode, short nBarCode);
	[DllImport(softekLibPath)]
	private static extern bool mtGetConvertUPCEToEAN13(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetConvertUPCEToEAN13(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern short mtScanBarCodeFromBitmapDump(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] strFilePath);
	[DllImport(softekLibPath)]
	private static extern short mtGetSkewTolerance(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetSkewTolerance(IntPtr hBarcode, short sNewValue);
	[DllImport(softekLibPath)]
	private static extern bool mtGetReadCode25ni(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetReadCode25ni(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern bool mtSetPhotometric(IntPtr hBarcode, bool bPhotometric);
	[DllImport(softekLibPath)]
	private static extern bool mtGetPhotometric(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetAllowDuplicateValues(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern bool mtGetAllowDuplicateValues(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetDespeckle(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern bool mtGetDespeckle(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtGetMinSeparation(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetMinSeparation(IntPtr hBarcode, short nNewValue);
	[DllImport(softekLibPath)]
	private static extern short mtGetBarStringQualityScore(IntPtr hBarcode, short nBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetExtendedCode39(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern bool mtGetExtendedCode39(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetCode39Checksum(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern bool mtGetCode39Checksum(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetErrorCorrection(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern bool mtGetErrorCorrection(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetReadNumeric(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern bool mtGetReadNumeric(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern IntPtr mtSetPattern(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] lpszNewValue);
	[DllImport(softekLibPath)]
	private static extern IntPtr mtGetPattern(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetPdfBpp(IntPtr hBarcode, short nNewValue);
	[DllImport(softekLibPath)]
	private static extern short mtGetPdfBpp(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetPdfDpi(IntPtr hBarcode, short nNewValue);
	[DllImport(softekLibPath)]
	private static extern short mtGetPdfDpi(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetMinSpaceBarWidth(IntPtr hBarcode, short nNewValue);
	[DllImport(softekLibPath)]
	private static extern short mtGetMinSpaceBarWidth(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtGetReadPDF417(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetReadPDF417(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern bool mtGetReadMicroPDF417(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetReadMicroPDF417(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern bool mtGetMedianFilter(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetMedianFilter(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern int mtSaveResults(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] filePath);
	[DllImport(softekLibPath)]
	private static extern int mtLoadXMLSettings(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] filePath, bool silent);
	[DllImport(softekLibPath)]
	private static extern int mtExportXMLSettings(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] filePath);
	[DllImport(softekLibPath)]
	private static extern int mtProcessXML(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] inputFilePath, [MarshalAs(UnmanagedType.LPArray)] byte[] outputFilePath, bool silent);
	[DllImport(softekLibPath)]
	private static extern bool mtGetCode25Checksum(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetCode25Checksum(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern short mtSetEncoding(IntPtr hBarcode, short nNewValue);
	[DllImport(softekLibPath)]
	private static extern short mtGetEncoding(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetGammaCorrection(IntPtr hBarcode, short nNewValue);
	[DllImport(softekLibPath)]
	private static extern short mtGetGammaCorrection(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetSkewLineJump(IntPtr hBarcode, short nNewValue);
	[DllImport(softekLibPath)]
	private static extern short mtGetSkewLineJump(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtGetReadDataMatrix(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetReadDataMatrix(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern bool mtGetReadQRCode(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetReadQRCode(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern bool mtGetReadDatabar(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetReadDatabar(IntPtr hBarcode, bool bNewValue);
	[DllImport(softekLibPath)]
	private static extern short mtGetColorProcessingLevel(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetColorProcessingLevel(IntPtr hBarcode, short value);
	[DllImport(softekLibPath)]
	private static extern short mtGetDatabarOptions(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetDatabarOptions(IntPtr hBarcode, short value);
	[DllImport(softekLibPath)]
	private static extern bool mtGetPdfImageOnly(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetPdfImageOnly(IntPtr hBarcode, bool value);
	[DllImport(softekLibPath)]
	private static extern bool mtGetReadCode93(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetReadCode93(IntPtr hBarcode, bool value);
	[DllImport(softekLibPath)]
	private static extern short mtGetPdfImageExtractOptions(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetPdfImageExtractOptions(IntPtr hBarcode, short value);
	[DllImport(softekLibPath)]
	private static extern short mtGetPdfImageRasterOptions(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetPdfImageRasterOptions(IntPtr hBarcode, short value);
	[DllImport(softekLibPath)]
	private static extern int getFunctionalityLevel();
	[DllImport(softekLibPath)]
	private static extern int mtGetLastError(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern int mtGetLastWarning(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern int mtGetLastWinError(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtGetUseFastScan(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetUseFastScan(IntPtr hBarcode, bool value);
	[DllImport(softekLibPath)]
	private static extern bool mtGetBarcodesAtTopOfPage(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetBarcodesAtTopOfPage(IntPtr hBarcode, bool value);
	[DllImport(softekLibPath)]
	private static extern short mtGetFastScanLineJump(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetFastScanLineJump(IntPtr hBarcode, short value);
	[DllImport(softekLibPath)]
	private static extern short mtGetMaxBarcodesPerPage(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetMaxBarcodesPerPage(IntPtr hBarcode, short value);
	[DllImport(softekLibPath)]
	private static extern bool mtGetSkewedDatamatrix(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetSkewedDatamatrix(IntPtr hBarcode, bool value);
	[DllImport(softekLibPath)]
	private static extern int mtGetTimeOut(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern int mtSetTimeOut(IntPtr hBarcode, int value);
	[DllImport(softekLibPath)]
	private static extern bool mtGetSkewedLinear(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetSkewedLinear(IntPtr hBarcode, bool value);
	[DllImport(softekLibPath)]
	private static extern short mtGetReportUnreadBarcodes(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetReportUnreadBarcodes(IntPtr hBarcode, short value);
	[DllImport(softekLibPath)]
	private static extern short mtGetShowCodabarStartStop(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetShowCodabarStartStop(IntPtr hBarcode, short value);
	[DllImport(softekLibPath)]
	private static extern short mtGetProgress(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtScanBarCodeInBackground(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] file);
	[DllImport(softekLibPath)]
	private static extern short mtScanBarCodeWait(IntPtr hBarcode, Int32 dwMilliSeconds);
	[DllImport(softekLibPath)]
	private static extern short mtGetBarCodeCount(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtScanBarCodeAbort(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtGetScanExitCode(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtGetMaxThreads(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetMaxThreads(IntPtr hBarcode, short value);
	[DllImport(softekLibPath)]
	private static extern bool mtGetRotateBy45IfNoBarcode(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtGetCode128DebugMode(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtGetCode128Lenient(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtGetCode128SearchLevel(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtGetMedianFilterBias(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtGetPdf417Debug(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtGetPdf417ChannelMode(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtGetPdfLocking(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtGetQrCodeReadInverted(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtGetQuotedPrintableCharSet(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtGetWeightLongerBarcodes(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtGetCode39MaxRatioPcnt(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtGetDataMatrixFinderGapTolerance(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtGetDataMatrixAutoUTF8(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtGetPatchCodeMinOccurrence(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtGetPdf417AutoUTF8(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtGetPdf417MacroEscapeBackslash(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtGetQRCodeAutoUTF8(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtGetQRCodeAutoMedianFilter(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtGetUseOldCode128Algorithm(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtGetUseRunCache(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetRotateBy45IfNoBarcode(IntPtr hBarcode, bool value);
	[DllImport(softekLibPath)]
	private static extern short mtSetCode128DebugMode(IntPtr hBarcode, short value);
	[DllImport(softekLibPath)]
	private static extern bool mtSetCode128Lenient(IntPtr hBarcode, bool value);
	[DllImport(softekLibPath)]
	private static extern short mtSetCode128SearchLevel(IntPtr hBarcode, short value);
	[DllImport(softekLibPath)]
	private static extern short mtSetMedianFilterBias(IntPtr hBarcode, short value);
	[DllImport(softekLibPath)]
	private static extern bool mtSetPdf417Debug(IntPtr hBarcode, bool value);
	[DllImport(softekLibPath)]
	private static extern short mtSetPdf417ChannelMode(IntPtr hBarcode, short value);
	[DllImport(softekLibPath)]
	private static extern short mtSetPdfLocking(IntPtr hBarcode, short value);
	[DllImport(softekLibPath)]
	private static extern bool mtSetQrCodeReadInverted(IntPtr hBarcode, bool value);
	[DllImport(softekLibPath)]
	private static extern short mtSetQuotedPrintableCharSet(IntPtr hBarcode, short value);
	[DllImport(softekLibPath)]
	private static extern bool mtSetWeightLongerBarcodes(IntPtr hBarcode, bool value);
	[DllImport(softekLibPath)]
	private static extern short mtSetCode39MaxRatioPcnt(IntPtr hBarcode, short value);
	[DllImport(softekLibPath)]
	private static extern short mtSetDataMatrixFinderGapTolerance(IntPtr hBarcode, short value);
	[DllImport(softekLibPath)]
	private static extern bool mtSetDataMatrixAutoUTF8(IntPtr hBarcode, bool value);
	[DllImport(softekLibPath)]
	private static extern short mtSetPatchCodeMinOccurrence(IntPtr hBarcode, short value);
	[DllImport(softekLibPath)]
	private static extern bool mtSetPdf417AutoUTF8(IntPtr hBarcode, bool value);
	[DllImport(softekLibPath)]
	private static extern bool mtSetPdf417MacroEscapeBackslash(IntPtr hBarcode, bool value);
	[DllImport(softekLibPath)]
	private static extern bool mtSetQRCodeAutoUTF8(IntPtr hBarcode, bool value);
	[DllImport(softekLibPath)]
	private static extern short mtSetQRCodeAutoMedianFilter(IntPtr hBarcode, short value);
	[DllImport(softekLibPath)]
	private static extern bool mtSetUseOldCode128Algorithm(IntPtr hBarcode, bool value);
	[DllImport(softekLibPath)]
	private static extern bool mtSetUseRunCache(IntPtr hBarcode, bool value);
	[DllImport(softekLibPath)]
	private static extern short mtSetDeskewMode(IntPtr hBarcode, short value);
	[DllImport(softekLibPath)]
	private static extern short mtGetDeskewMode(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern IntPtr mtSetPdfPassword(IntPtr hBarcode, [MarshalAs(UnmanagedType.LPArray)] byte[] lpszNewValue);
	[DllImport(softekLibPath)]
	private static extern IntPtr mtGetPdfPassword(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetDataMatrixSearchLevel(IntPtr hBarcode, short value);
	[DllImport(softekLibPath)]
	private static extern short mtGetDataMatrixSearchLevel(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern short mtSetDataMatrixRectangleSupport(IntPtr hBarcode, short value);
	[DllImport(softekLibPath)]
	private static extern short mtGetDataMatrixRectangleSupport(IntPtr hBarcode);
	[DllImport(softekLibPath)]
	private static extern bool mtSetReadEAN13Supplemental(IntPtr hBarcode, bool value);
	[DllImport(softekLibPath)]
	private static extern bool mtGetReadEAN13Supplemental(IntPtr hBarcode);


	public int LineJump ;
	public int MinOccurrence ;
	public int PrefOccurrence ;
	public bool MultipleRead ;
	public bool MedianFilter ;
	public int NoiseReduction ;
	public bool Despeckle ;
	public int QuietZoneSize ;
	public int SkewTolerance ;
	public int PageNo ;
	public bool ReadCode39 ;
        public bool ReadCode93;
        public bool ReadCode128;
		public bool ReadCode25 ;
		public bool ReadCode25ni ;
		public bool AllowDuplicateValues ;
		public bool ReadEAN8 ;
		public bool ReadEAN13 ;
		public bool ReadUPCA ;
		public bool ReadUPCE ;
		public bool ReadCodabar ;
		public bool ReadPatchCodes ;
		public bool ShowCheckDigit ;
		public bool Code39NeedStartStop ;
		public bool UseOverSampling ;
		public int ScanDirection ;
		public bool ConvertUPCEToEAN13 ;
		public int MinLength ;
		public int MaxLength ;
		public int ColorThreshold ;
		public int MinSeparation ;	
		public bool Code39Checksum ;
		public bool ExtendedCode39 ;
		public bool ErrorCorrection ;
		public bool ReadNumeric ;
		public string Pattern ;
		public string TifSplitPath ;
		public int	TifSplitMode ;
		public int	MinSpaceBarWidth ;
		public bool	LicensedCopy ;
		public bool ReadPDF417 ;
		public bool Photometric ;
		public string message ;
		public int PdfBpp ;
		public int PdfDpi ;
		public string build ;
		public int SkewLineJump ;
		public int GammaCorrection ;
		public int Encoding ;
		public bool Code25Checksum ;
		public bool ReadMicroPDF417 ;
		public int ShortCode128MinLength ;
		public bool ReadShortCode128 ;
		public string LicenseKey ;
		public bool ConvertPDFToTIF ;
		public bool ReadDataMatrix ;
        public bool ReadQRCode;
        public bool ReadDatabar;
        public bool PdfImageOnly;
		public int DatabarOptions ;
		public int ColorProcessingLevel ;
        public int PdfImageExtractOptions;
        public int PdfImageRasterOptions;
        public bool UseFastScan;
        public bool BarcodesAtTopOfPage;
        public int FastScanLineJump;
        public int MaxBarcodesPerPage;
        public bool SkewedDatamatrix;
        public int TimeOut;
        public bool SkewedLinear;
        public int ReportUnreadBarcodes;
        public int ShowCodabarStartStop;
        public bool dllLoaded;
        public int MaxThreads;
        public bool RotateBy45IfNoBarcode;
        public int Code128DebugMode;
        public bool Code128Lenient;
        public int Code128SearchLevel;
        public int MedianFilterBias;
        public bool Pdf417Debug;
        public int Pdf417ChannelMode;
        public int PdfLocking;
        public bool QrCodeReadInverted;
        public int QuotedPrintableCharSet;
        public bool WeightLongerBarcodes;
        public int Code39MaxRatioPcnt;
        public int DataMatrixFinderGapTolerance;
        public bool DataMatrixAutoUTF8;
        public int PatchCodeMinOccurrence;
        public bool Pdf417AutoUTF8;
        public bool Pdf417MacroEscapeBackslash;
        public bool QRCodeAutoUTF8;
        public short QRCodeAutoMedianFilter;
        public bool UseOldCode128Algorithm;
        public bool UseRunCache;
        public int DeskewMode;
        public string PdfPassword;
        public int DataMatrixSearchLevel;
        public int DataMatrixRectangleSupport;
        public bool ReadEAN13Supplemental;

        private IntPtr hBarcode;
        private string pdflib = "";
        private IntPtr hSoftekBarcodeDLL;
        private System.Text.UTF8Encoding encoding ;

        private bool disposed = false;

        //private string[] pdfLibs = { };
        private IntPtr pdfLibHandle;


        public BarcodeReader()
        {
            hBarcode = hSoftekBarcodeDLL = IntPtr.Zero;
            encoding = new System.Text.UTF8Encoding();
            initBarcodeReader();
        }

        private void initBarcodeReader()
        {
            message = "";
            Pattern = "";
            LicenseKey = "";
            TifSplitPath = "";
            TifSplitMode = 0;

            hBarcode = mtCreateBarcodeInstance();
            GetProperties();
            LoadXMLSettings("<FilePathEncoding>1</>");
            Photometric = true;
        }

        ~ BarcodeReader()
        {
            Dispose(false);
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        protected virtual void Dispose(bool disposeManagedResources)
        {
            // process only if mananged and unmanaged resources have
            // not been disposed of.
            if (!this.disposed)
            {
                disposed = true;
                
                // dispose unmanaged resources
                // Trace.WriteLine("BarcodeReader: Disposing unmanaged resouces");

                if (hBarcode != IntPtr.Zero)
                    mtDestroyBarcodeInstance(hBarcode);

                {
                    //Trace.WriteLine("BarcodeReader: Disposing managed resources");
                    pdfLibHandle = IntPtr.Zero;
                    encoding = null;
                }

                disposed = true;
            }
            else
            {
                ;// Trace.WriteLine("BarcodeReader: Resources already disposed");
            }
        }

        private byte[] StrToByteArray(string str)
        {
            return System.Text.Encoding.UTF8.GetBytes(str);
            //return encoding.GetBytes(str);
        }

        public int GetProgress()
        {
            return mtGetProgress(hBarcode);
        }



        public int GetLastError()
        {
            return mtGetLastError(hBarcode);
        }

        public int GetLastWarning()
        {
            return mtGetLastWarning(hBarcode);
        }


        public int GetLastWinError()
        {
            return mtGetLastWinError(hBarcode);
        }

        private bool testFileAccess(string filePath)
        {
            try
            {
                File.Open(filePath, FileMode.Open, FileAccess.Read).Dispose();
                return true;
            }
            catch (IOException)
            {
                return false;
            }
        }

		public int ScanBarCode(string strFilePath)
        {
            if (strFilePath.EndsWith(".pdf", StringComparison.CurrentCultureIgnoreCase))
            {
                if (pdfLibHandle == IntPtr.Zero)
                    throw new System.DllNotFoundException("The following dll file needed for processing pdf documents could not be loaded: " + pdflib);
            }
		    SetProperties() ;
            //if (!testFileAccess(strFilePath))
            //    return -999;

            return mtScanBarCode(hBarcode, StrToByteArray(strFilePath)) ;
        }

        public int ScanBarCodeInBackground(string strFilePath)
        {
            if (strFilePath.EndsWith(".pdf", StringComparison.CurrentCultureIgnoreCase))
            {
                if (pdfLibHandle == IntPtr.Zero)
                    throw new System.DllNotFoundException("The following dll file needed for processing pdf documents could not be loaded: " + pdflib);
            }
            SetProperties();
            return mtScanBarCodeInBackground(hBarcode, StrToByteArray(strFilePath));
        }

        public int ScanBarCodeAbort()
        {
            return mtScanBarCodeAbort(hBarcode);
        }

        public int GetScanExitCode()
        {
            return mtGetScanExitCode(hBarcode);
        }

        // -1 means infinite, 0 is just a check 
        public int ScanBarCodeWait(int milliSeconds)
        {
            return mtScanBarCodeWait(hBarcode, milliSeconds); 
        }

        public int GetBarCodeCount()
        {
            return mtGetBarCodeCount(hBarcode);
        }

        public int GetBarStringQualityScore(int nBarcode)
        {
            return mtGetBarStringQualityScore(hBarcode, (short)nBarcode);
        }

        public byte[] GetBarStringBytes(int nBarcode)
        {
            string ansiStr = System.Runtime.InteropServices.Marshal.PtrToStringAnsi(mtGetBarString(hBarcode, (short)nBarcode));
            System.Text.Encoding iso = System.Text.Encoding.GetEncoding("ISO-8859-1");
            return iso.GetBytes(ansiStr);
        }


        public byte[] GetRawBarStringBytes(int nBarcode)
        {
            // Get the required size 
            int sz = mtGetRawBarString(hBarcode, (short)nBarcode, null, 0);

            if (sz < 0) return new byte[0];

            // Now get the bytes for real
            byte[] b = new byte[sz];
            sz = mtGetRawBarString(hBarcode, (short)nBarcode, b, b.Length);

            if (sz < 0) return new byte[0];

            return b;
        }

		public string GetBarString(int nBarcode)
        {
            if (Encoding == 3) // UTF-8 encoding (the default)
            {
                int bufSize = 10000;

                byte[] buffer = new byte[bufSize];

                int l = mtGetBarStringA(hBarcode, (short)nBarcode, buffer, bufSize);

                if (l > 0)
                    return System.Text.Encoding.UTF8.GetString(buffer, 0, l);
                else
                    return "";
            }
            else
            {
                return System.Runtime.InteropServices.Marshal.PtrToStringAnsi(mtGetBarString(hBarcode, (short)nBarcode));
            }
        }


		public string GetBarStringType(int nBarcode)
        {
            return System.Runtime.InteropServices.Marshal.PtrToStringAnsi(mtGetBarStringType(hBarcode, (short) nBarcode));
        }

		public System.Drawing.Rectangle GetBarStringRect(int nBarcode)
        {
            int tlx, tly, brx, bry ;
            tlx = mtGetBarStringTlx(hBarcode, (short) nBarcode);
            tly = mtGetBarStringTly(hBarcode, (short)nBarcode);
            brx = mtGetBarStringBrx(hBarcode, (short)nBarcode);
            bry = mtGetBarStringBry(hBarcode, (short)nBarcode);

            System.Drawing.Rectangle rect = new Rectangle(tlx, tly, brx - tlx, bry - tly);
            return rect ;
        }

		public int SetScanRect(System.Drawing.Rectangle rect)
        {
            return SetScanRect(rect, 0);
        }

		public int SetScanRect(System.Drawing.Rectangle rect, int MappingMode)
        {
            if (mtSetScanRectA(hBarcode, (Int32)rect.Left, (Int32)rect.Top, (Int32)rect.Right, (Int32)rect.Bottom, (short)MappingMode))
                return 1;
            return 0;
        }

		public int GetBarStringDirection(int nBarCode)
        {
            return mtGetBarStringDirection(hBarcode, (short) nBarCode) ;
        }

		public int GetBarStringPage(int n)
        {
            return mtGetBarStringPage(hBarcode, (short)n);
        }

/*
		public int ScanBarCodeFromBitmap(System.Drawing.Bitmap bmp)
        {
            SetProperties();

            System.Drawing.Imaging.PixelFormat pixelFormat = bmp.PixelFormat;

            short bitsPerPel;

            if (bmp.PixelFormat == System.Drawing.Imaging.PixelFormat.Format1bppIndexed)
            {
                bitsPerPel = 1;
            }
            else if (bmp.PixelFormat == System.Drawing.Imaging.PixelFormat.Format8bppIndexed)
            {
                bitsPerPel = 8;
            }
            else
            {
                bitsPerPel = 24;
                pixelFormat = System.Drawing.Imaging.PixelFormat.Format24bppRgb;
            }
            
            System.Drawing.Rectangle rect = new System.Drawing.Rectangle(0, 0, bmp.Width, bmp.Height);
            System.Drawing.Imaging.BitmapData bmData = bmp.LockBits(rect, System.Drawing.Imaging.ImageLockMode.ReadOnly,pixelFormat) ;

            int retval ;

            if ((bmData.Stride > 0) && ((bmData.Stride % 4) == 0))
            {
                retval = mtScanBarCodeFromBitmapA(hBarcode, 0, bmData.Width, bmData.Height, bmData.Stride, 1, bitsPerPel, bmData.Scan0);
                bmp.UnlockBits(bmData);
            }
            else
            {
                IntPtr hBitmap = bmp.GetHbitmap();

                if (hBitmap == IntPtr.Zero)
                {
                    bmp.UnlockBits(bmData);
                    return -1;
                }

                retval = mtScanBarCodeFromBitmap(hBarcode, hBitmap);

                DeleteObject(hBitmap);
            }
            return retval;
        }

		public int ScanBarCodeFromBitmap(IntPtr hBitmap)
        {
            SetProperties();
            return mtScanBarCodeFromBitmap(hBarcode, hBitmap);
        }

		public int ScanBarCodeFromDIB (IntPtr hDIB)
        {
            SetProperties();
            return mtScanBarCodeFromDIB(hBarcode, hDIB);
        }
*/

		public bool ProcessXML(string strInputFilePath, string strOutputFilePath)
        {
            return ProcessXML(strInputFilePath, strOutputFilePath, true) ;
        }

        public bool ProcessXML(string strInputFilePath, string strOutputFilePath, bool silent)
        {
            SetProperties();
            bool retval = Convert.ToBoolean(mtProcessXML(hBarcode, StrToByteArray(strInputFilePath), StrToByteArray(strOutputFilePath), silent));
            GetProperties();
            return retval;
        }

		public bool ExportXMLSettings(string strFilePath)
        {
            SetProperties();
            bool retval = Convert.ToBoolean(mtExportXMLSettings(hBarcode, StrToByteArray(strFilePath)));
            GetProperties();
            return retval;
        }

		public bool SaveResults(string strFilePath)
        {
            SetProperties();
            bool retval = Convert.ToBoolean(mtSaveResults(hBarcode, StrToByteArray(strFilePath)));
            GetProperties();
            return retval;
        }

		public bool LoadXMLSettings(string strFilePath)
        {
            return LoadXMLSettings(strFilePath, true) ;
        }

        public bool LoadXMLSettings(string strFilePath, bool silent)
        {
            SetProperties();
            bool retval = Convert.ToBoolean(mtLoadXMLSettings(hBarcode, StrToByteArray(strFilePath), silent));
            GetProperties();
            return retval;
        }

        private void GetProperties()
        {
            MinOccurrence = mtGetMinOccurrence(hBarcode);
            LineJump = mtGetLineJump(hBarcode);
            PrefOccurrence = mtGetPrefOccurrence(hBarcode);
            MultipleRead = Convert.ToBoolean(mtGetMultipleRead(hBarcode));
            NoiseReduction = mtGetNoiseReduction(hBarcode);
            Despeckle = mtGetDespeckle(hBarcode);
            QuietZoneSize = mtGetQuietZoneSize(hBarcode);
            ReadCode39 = Convert.ToBoolean(mtGetReadCode39(hBarcode));
            ReadCode93 = Convert.ToBoolean(mtGetReadCode93(hBarcode));
            ReadPDF417 = Convert.ToBoolean(mtGetReadPDF417(hBarcode));
            ReadCode128 = Convert.ToBoolean(mtGetReadCode128(hBarcode));
            ReadCode25 = Convert.ToBoolean(mtGetReadCode25(hBarcode));
            ReadCode25ni = Convert.ToBoolean(mtGetReadCode25ni(hBarcode));
            ReadEAN8 = Convert.ToBoolean(mtGetReadEAN8(hBarcode));
            AllowDuplicateValues = Convert.ToBoolean(mtGetAllowDuplicateValues(hBarcode));
            ReadEAN13 = Convert.ToBoolean(mtGetReadEAN13(hBarcode));
            ReadUPCA = Convert.ToBoolean(mtGetReadUPCA(hBarcode));
            ReadUPCE = Convert.ToBoolean(mtGetReadUPCE(hBarcode));
            ReadCodabar = Convert.ToBoolean(mtGetReadCodabar(hBarcode));
            ReadPatchCodes = Convert.ToBoolean(mtGetReadPatchCodes(hBarcode));
            ConvertUPCEToEAN13 = Convert.ToBoolean(mtGetConvertUPCEToEAN13(hBarcode));
            ShowCheckDigit = Convert.ToBoolean(mtGetShowCheckDigit(hBarcode));
            Code39NeedStartStop = Convert.ToBoolean(mtGetCode39NeedStartStop(hBarcode));
            ColorThreshold = mtGetColorThreshold(hBarcode);
            UseOverSampling = Convert.ToBoolean(mtGetUseOverSampling(hBarcode));
            ScanDirection = mtGetScanDirection(hBarcode);
            MinLength = mtGetMinLength(hBarcode);
            MaxLength = mtGetMaxLength(hBarcode);
            SkewTolerance = mtGetSkewTolerance(hBarcode);
            Code39Checksum = Convert.ToBoolean(mtGetCode39Checksum(hBarcode));
            ExtendedCode39 = Convert.ToBoolean(mtGetExtendedCode39(hBarcode));
            ErrorCorrection = Convert.ToBoolean(mtGetErrorCorrection(hBarcode));
            MedianFilter = Convert.ToBoolean(mtGetMedianFilter(hBarcode));
            ReadNumeric = Convert.ToBoolean(mtGetReadNumeric(hBarcode));
            MinSeparation = mtGetMinSeparation(hBarcode);
            SkewLineJump = mtGetSkewLineJump(hBarcode);
            GammaCorrection = mtGetGammaCorrection(hBarcode);
            Encoding = mtGetEncoding(hBarcode);
            Code25Checksum = Convert.ToBoolean(mtGetCode25Checksum(hBarcode));
            PageNo = mtGetPageNo(hBarcode);
            PdfBpp = mtGetPdfBpp(hBarcode);
            PdfDpi = mtGetPdfDpi(hBarcode);
            Photometric = mtGetPhotometric(hBarcode);
            MinSpaceBarWidth = mtGetMinSpaceBarWidth(hBarcode);
            ReadMicroPDF417 = Convert.ToBoolean(mtGetReadMicroPDF417(hBarcode));
            ReadShortCode128 = Convert.ToBoolean(mtGetReadShortCode128(hBarcode));
            ShortCode128MinLength = mtGetShortCode128MinLength(hBarcode);
            ReadDataMatrix = Convert.ToBoolean(mtGetReadDataMatrix(hBarcode));
            ReadQRCode = Convert.ToBoolean(mtGetReadQRCode(hBarcode));
            ReadDatabar = Convert.ToBoolean(mtGetReadDatabar(hBarcode));
            DatabarOptions = mtGetDatabarOptions(hBarcode);
            ColorProcessingLevel = mtGetColorProcessingLevel(hBarcode);
            PdfImageOnly = Convert.ToBoolean(mtGetPdfImageOnly(hBarcode));
            PdfImageExtractOptions = mtGetPdfImageExtractOptions(hBarcode);
            PdfImageRasterOptions = mtGetPdfImageRasterOptions(hBarcode);
            UseFastScan = mtGetUseFastScan(hBarcode);
            BarcodesAtTopOfPage = mtGetBarcodesAtTopOfPage(hBarcode);
            FastScanLineJump = mtGetFastScanLineJump(hBarcode);
            MaxBarcodesPerPage = mtGetMaxBarcodesPerPage(hBarcode);
            SkewedDatamatrix = mtGetSkewedDatamatrix(hBarcode);
            TimeOut = mtGetTimeOut(hBarcode);
            SkewedLinear = mtGetSkewedLinear(hBarcode);
            ReportUnreadBarcodes = mtGetReportUnreadBarcodes(hBarcode);
            ShowCodabarStartStop = mtGetShowCodabarStartStop(hBarcode);
            Pattern = System.Runtime.InteropServices.Marshal.PtrToStringAnsi(mtGetPattern(hBarcode));
            MaxThreads = mtGetMaxThreads(hBarcode);
            RotateBy45IfNoBarcode = mtGetRotateBy45IfNoBarcode(hBarcode);
            Code128DebugMode = mtGetCode128DebugMode(hBarcode);
            Code128Lenient = mtGetCode128Lenient(hBarcode);
            Code128SearchLevel = mtGetCode128SearchLevel(hBarcode);
            MedianFilterBias = mtGetMedianFilterBias(hBarcode);
            Pdf417Debug = mtGetPdf417Debug(hBarcode);
            Pdf417ChannelMode = mtGetPdf417ChannelMode(hBarcode);
            PdfLocking = mtGetPdfLocking(hBarcode);
            QrCodeReadInverted = mtGetQrCodeReadInverted(hBarcode);
            QuotedPrintableCharSet = mtGetQuotedPrintableCharSet(hBarcode);
            WeightLongerBarcodes = mtGetWeightLongerBarcodes(hBarcode);
            Code39MaxRatioPcnt = mtGetCode39MaxRatioPcnt(hBarcode);
            DataMatrixFinderGapTolerance = mtGetDataMatrixFinderGapTolerance(hBarcode);
            DataMatrixAutoUTF8 = mtGetDataMatrixAutoUTF8(hBarcode);
            PatchCodeMinOccurrence = mtGetPatchCodeMinOccurrence(hBarcode);
            Pdf417AutoUTF8 = mtGetPdf417AutoUTF8(hBarcode);
            Pdf417MacroEscapeBackslash = mtGetPdf417MacroEscapeBackslash(hBarcode);
            QRCodeAutoUTF8 = mtGetQRCodeAutoUTF8(hBarcode);
            QRCodeAutoMedianFilter = mtGetQRCodeAutoMedianFilter(hBarcode);
            UseOldCode128Algorithm = mtGetUseOldCode128Algorithm(hBarcode);
            TifSplitMode = mtGetTifSplitMode(hBarcode);
            TifSplitPath = System.Runtime.InteropServices.Marshal.PtrToStringAnsi(mtGetTifSplitPath(hBarcode));
            UseRunCache = mtGetUseRunCache(hBarcode);
            DeskewMode = mtGetDeskewMode(hBarcode);
            PdfPassword = System.Runtime.InteropServices.Marshal.PtrToStringAnsi(mtGetPdfPassword(hBarcode));
            DataMatrixSearchLevel = mtGetDataMatrixSearchLevel(hBarcode);
            DataMatrixRectangleSupport = mtGetDataMatrixRectangleSupport(hBarcode);
            ReadEAN13Supplemental = mtGetReadEAN13Supplemental(hBarcode);

        }

	    private void SetProperties()
	    {
		    mtSetMinOccurrence(hBarcode, (short) MinOccurrence);
            mtSetLineJump(hBarcode, (short)LineJump);
            mtSetPrefOccurrence(hBarcode, (short)PrefOccurrence);
            mtSetMultipleRead(hBarcode, MultipleRead);
		    mtSetNoiseReduction(hBarcode, (short) NoiseReduction);
		    mtSetDespeckle(hBarcode, Despeckle);
		    mtSetQuietZoneSize(hBarcode, (short) QuietZoneSize);
            mtSetReadCode39(hBarcode, ReadCode39);
            mtSetReadCode93(hBarcode, ReadCode93);
            mtSetReadPDF417(hBarcode, ReadPDF417);
            mtSetReadCode128(hBarcode, ReadCode128);
            mtSetReadCode25(hBarcode, ReadCode25);
            mtSetReadCode25ni(hBarcode, ReadCode25ni);
            mtSetReadEAN8(hBarcode, ReadEAN8);
            mtSetAllowDuplicateValues(hBarcode, AllowDuplicateValues);
            mtSetReadEAN13(hBarcode, ReadEAN13);
            mtSetReadUPCA(hBarcode, ReadUPCA);
            mtSetReadUPCE(hBarcode, ReadUPCE);
            mtSetReadCodabar(hBarcode, ReadCodabar);
            mtSetReadPatchCodes(hBarcode, ReadPatchCodes);
            mtSetConvertUPCEToEAN13(hBarcode, ConvertUPCEToEAN13);
            mtSetShowCheckDigit(hBarcode, ShowCheckDigit);
            mtSetCode39NeedStartStop(hBarcode, Code39NeedStartStop);
		    mtSetColorThreshold(hBarcode, (short) ColorThreshold);
            mtSetUseOverSampling(hBarcode, UseOverSampling);
		    mtSetScanDirection(hBarcode, (short) ScanDirection);
		    mtSetMinLength(hBarcode, (short) MinLength);
		    mtSetMaxLength(hBarcode, (short) MaxLength);
		    mtSetSkewTolerance(hBarcode, (short) SkewTolerance);
            mtSetCode39Checksum(hBarcode, Code39Checksum);
            mtSetExtendedCode39(hBarcode, ExtendedCode39);
            mtSetErrorCorrection(hBarcode, ErrorCorrection);
            mtSetMedianFilter(hBarcode, MedianFilter);
            mtSetReadNumeric(hBarcode, ReadNumeric);
		    mtSetMinSeparation(hBarcode, (short) MinSeparation);
		    mtSetMinSpaceBarWidth(hBarcode, (short) MinSpaceBarWidth);
		    mtSetSkewLineJump(hBarcode, (short) SkewLineJump);
		    mtSetGammaCorrection(hBarcode, (short) GammaCorrection);
		    mtSetEncoding(hBarcode, (short) Encoding);
		    mtSetCode25Checksum(hBarcode, Code25Checksum);
		    mtSetPageNo(hBarcode, (short) PageNo);
		    mtSetPdfBpp(hBarcode, (short) PdfBpp);
		    mtSetPdfDpi(hBarcode, (short) PdfDpi);
		    mtSetPhotometric(hBarcode, Photometric);
            mtSetReadMicroPDF417(hBarcode, ReadMicroPDF417);
            mtSetReadShortCode128(hBarcode, ReadShortCode128);
		    mtSetShortCode128MinLength(hBarcode, (short) ShortCode128MinLength);
            mtSetReadDataMatrix(hBarcode, ReadDataMatrix);
            mtSetReadQRCode(hBarcode, ReadQRCode);
            mtSetReadDatabar(hBarcode, ReadDatabar);
		    mtSetDatabarOptions(hBarcode, (short) DatabarOptions);
		    mtSetColorProcessingLevel(hBarcode, (short) ColorProcessingLevel);
            mtSetTifSplitMode(hBarcode, (short) TifSplitMode);
            mtSetPdfImageOnly(hBarcode, PdfImageOnly);
            mtSetPdfImageExtractOptions(hBarcode, (short)PdfImageExtractOptions);
            mtSetPdfImageRasterOptions(hBarcode, (short)PdfImageRasterOptions);
            mtSetUseFastScan(hBarcode, UseFastScan);
            mtSetBarcodesAtTopOfPage(hBarcode, BarcodesAtTopOfPage);
            mtSetFastScanLineJump(hBarcode, (short) FastScanLineJump);
            mtSetMaxBarcodesPerPage(hBarcode, (short) MaxBarcodesPerPage);
            mtSetSkewedDatamatrix(hBarcode, SkewedDatamatrix);
            mtSetTimeOut(hBarcode, TimeOut);
            mtSetSkewedLinear(hBarcode, SkewedLinear);
            mtSetReportUnreadBarcodes(hBarcode, (short) ReportUnreadBarcodes);
            mtSetShowCodabarStartStop(hBarcode, (short) ShowCodabarStartStop);
            mtSetMaxThreads(hBarcode, (short)MaxThreads);
            mtSetRotateBy45IfNoBarcode(hBarcode, RotateBy45IfNoBarcode);
            mtSetCode128DebugMode(hBarcode, (short)Code128DebugMode);
            mtSetCode128Lenient(hBarcode, Code128Lenient);
            mtSetCode128SearchLevel(hBarcode, (short)Code128SearchLevel);
            mtSetMedianFilterBias(hBarcode, (short)MedianFilterBias);
            mtSetPdf417Debug(hBarcode, Pdf417Debug);
            mtSetPdf417ChannelMode(hBarcode, (short)Pdf417ChannelMode);
            mtSetPdfLocking(hBarcode, (short)PdfLocking);
            mtSetQrCodeReadInverted(hBarcode, QrCodeReadInverted);
            mtSetQuotedPrintableCharSet(hBarcode, (short)QuotedPrintableCharSet);
            mtSetWeightLongerBarcodes(hBarcode, WeightLongerBarcodes);
            mtSetCode39MaxRatioPcnt(hBarcode, (short)Code39MaxRatioPcnt);
            mtSetDataMatrixFinderGapTolerance(hBarcode, (short)DataMatrixFinderGapTolerance);
            mtSetDataMatrixAutoUTF8(hBarcode, DataMatrixAutoUTF8);
            mtSetPatchCodeMinOccurrence(hBarcode, (short)PatchCodeMinOccurrence);
            mtSetPdf417AutoUTF8(hBarcode, Pdf417AutoUTF8);
            mtSetPdf417MacroEscapeBackslash(hBarcode, Pdf417MacroEscapeBackslash);
            mtSetQRCodeAutoUTF8(hBarcode, QRCodeAutoUTF8);
            mtSetQRCodeAutoMedianFilter(hBarcode, QRCodeAutoMedianFilter);
            mtSetUseOldCode128Algorithm(hBarcode, UseOldCode128Algorithm);
            mtSetUseRunCache(hBarcode, UseRunCache);
            mtSetDeskewMode(hBarcode, (short)DeskewMode);
            mtSetDataMatrixSearchLevel(hBarcode, (short)DataMatrixSearchLevel);
            mtSetDataMatrixRectangleSupport(hBarcode, (short)DataMatrixRectangleSupport);
            mtSetReadEAN13Supplemental(hBarcode, ReadEAN13Supplemental);

    	
            byte[] byteStr = StrToByteArray(Pattern);
		    mtSetPattern(hBarcode, byteStr) ;

            byteStr = StrToByteArray(TifSplitPath) ;
		    mtSetTifSplitPath(hBarcode, byteStr) ;

            byteStr = StrToByteArray(LicenseKey) ;
            mtSetLicenseKey(hBarcode, byteStr) ;

            byteStr = StrToByteArray(PdfPassword);
            mtSetPdfPassword(hBarcode, byteStr);

		    mtSetTifSplitMode(hBarcode, (short) TifSplitMode) ;
	    }

    }
}
